/*
 * =====================================================================================
 *
 *	This file is part of qKismet, Qt Kismet GUI.
 *
 *	qKismet is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	qKismet is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * =====================================================================================
 *
 *       Filename:  alertinfo.cpp
 *
 *    Description:  Alert informations
 *
 *        Created:  2009-09-19 19:13:57 CEST
 *
 *	Copyright (C) 2009 Krzysztof Tyszecki <krzychoocpp@gmail.com>
 *
 * =====================================================================================
 */

#include "alertinfo.h"
#include "kisock.h"
#include <QDateTime>

QHash<alertOption,QString alertInfo::*> alertInfo::optionToMember;
QHash<alertOption,QString> alertManager::optionToAlert, alertManager::optionToName;

using namespace AlertOptions;

uint qHash(const alertOption& opt)
{
        return qHash((int)opt);
}


alertInfo::alertInfo(kismetResponse &response, QSet<alertOption> &options)
{
        if(optionToMember.size() == 0)
                fillArray();

        foreach(alertOption opt, options)
                this->*optionToMember[opt]   = response.read().toString();

        if(options.contains(Sec))
        {
                QDateTime t;

                t    = QDateTime::fromTime_t(sec.toInt());
                if(options.contains(Usec))
                        t    = t.addMSecs(usec.toInt()/1000);
                time    = t.toString();
        }
}

void alertInfo::fillArray()
{
        optionToMember[Sec]     = &alertInfo::sec;
        optionToMember[Usec]    = &alertInfo::usec;
        optionToMember[Header]  = &alertInfo::header;
        optionToMember[Bssid]   = &alertInfo::bssid;
        optionToMember[Source]  = &alertInfo::source;
        optionToMember[Dest]    = &alertInfo::dest;
        optionToMember[Other]   = &alertInfo::channel;
        optionToMember[Text]    = &alertInfo::text;
        optionToMember[Time]    = &alertInfo::time;
}

alertInfo& alertInfo::operator |=(const alertInfo &info)
{
        foreach(alertOption opt,optionToMember.keys())
        {
                QString &oldv   = this->*optionToMember[opt];
                const QString &newv   = info.*optionToMember[opt];
                if(newv != "" && newv != oldv)
                        oldv       = newv;

        }
        return *this;
}

alertManager::alertManager(QKismetSocket *socket)
{
        sock    = socket;
        capabilitiesChanged("ALERT");
        connect(sock->protoManager(),SIGNAL(capsChanged(QString)),this,SLOT(capabilitiesChanged(QString)));
        sock->registerProtocol(this,"ALERT");
        initArrays();
}

alertManager* alertManager::manager(QKismetSocket *socket)
{
        alertManager *ret        = (alertManager*)socket->managerForProtocol("ALERT");

        return (ret) ? ret : new alertManager(socket);
}

void alertManager::parseData(const QString& protocol, kismetResponse &response)
{
        emit data(alertInfo(response,options));
}

QString alertManager::optionName(alertOption opt)
{
        return optionToName[opt];
}

QString alertManager::optionName(protocolOption opt)
{
        return optionToName[opt];
}

void alertManager::setOption(protocolOption option, bool enable)
{
        if(option >= INVALID)
                return;
        if(enable && !options.contains(option))
        {
                if((alertOption)option == Time)
                {
                        options |= Sec;
                        options |= Usec;
                }
                else
                        options |= option;
                update();
                emit activeChanged();
        }
        else if(!enable && options.contains(option))
        {
                if((alertOption)option == Time)
                {
                        options.remove(Sec);
                        options.remove(Usec);
                }
                else
                        options.remove(option);
                update();
                emit activeChanged();
        }
}

void alertManager::update()
{
        if(!sock->isWritable())
                return;

        QString cmd;

        if(options.size() > 0)
        {
                cmd     = "ENABLE ALERT ";

                QSet<alertOption>::const_iterator it = options.constBegin();
                while(it != options.constEnd())
                {
                        if(available.contains(*it))
                        {
                                cmd     += optionToAlert[*it];
                                cmd     += ',';
                        }
                        it++;
                }
                cmd.chop(1);
        }
        else
        {
                cmd     = "REMOVE ALERT";
        }
        qDebug() << cmd;
        sock->sendCommand(cmd);
}

void alertManager::initArrays()
{
        optionToAlert[Sec]     = "sec";
        optionToAlert[Usec]    = "usec";
        optionToAlert[Header]  = "header";
        optionToAlert[Bssid]   = "bssid";
        optionToAlert[Source]  = "source";
        optionToAlert[Dest]    = "dest";
        optionToAlert[Other]   = "channel";
        optionToAlert[Text]    = "text";

        optionToName[Sec]     = "Sec";
        optionToName[Usec]    = "µS";
        optionToName[Header]  = "Header";
        optionToName[Bssid]   = "BSSID";
        optionToName[Source]  = "Source";
        optionToName[Dest]    = "Destination";
        optionToName[Other]   = "Channel";
        optionToName[Text]    = "Text";
        optionToName[Time]    = "Time";
}

void alertManager::socketReady()
{
        update();
}

void alertManager::capabilitiesChanged(const QString& protocol)
{
        if(protocol == "ALERT")
        {
                alertOption opt;
                QStringList fields   = sock->protoManager()->caps(protocol);

                available.clear();
                foreach(QString o,fields)
                if((opt = optionToAlert.key(o,INVALID)) != INVALID)
                        available |= opt;

                update();

                emit dataManager::capabilitiesChanged();
        }
}

const QSet<alertOption>& alertManager::enabledOptions()
{
        if(options.contains(Sec) && options.contains(Usec))
        {
                optionsout  = options;
                optionsout.remove(Sec);
                optionsout.remove(Usec);
                optionsout       |= Time;
                return optionsout;
        }
        return options;
}

bool alertManager::isOptionEnabled(protocolOption opt)
{
        if((alertOption)opt == Time)
                return options.contains(Sec) && options.contains(Usec);
        return options.contains(opt);
}

const QSet<protocolOption>& alertManager::availableOptions()
{
        if(available.contains(Sec) && available.contains(Usec))
        {
                avail  = available;
                avail.remove(Sec);
                avail.remove(Usec);
                avail       |= Time;
                return avail;
        }
        return available;
}
