/*
 * =====================================================================================
 *
 *	This file is part of qKismet, Qt Kismet GUI.
 *
 *	qKismet is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	qKismet is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * =====================================================================================
 *
 *       Filename:  alertinfo.h
 *
 *    Description:  Alert informations
 *
 *        Created:  2009-09-19 19:13:57 CEST
 *
 *	Copyright (C) 2009 Krzysztof Tyszecki <krzychoocpp@gmail.com>
 *
 * =====================================================================================
 */

#ifndef ALERTINFO_H
#define ALERTINFO_H

#include "datamanager.h"
#include "protocolmanager.h"


class QKismetSocket;

namespace AlertOptions
{
        enum alertOptionEnum{Time,Sec,Usec,Header,Bssid,Source,Dest,Other,Channel,Text,INVALID};
}

class alertOption : public protocolOption
{
public:
        alertOption(const protocolOption &en){opt = (int)en;}
        alertOption(AlertOptions::alertOptionEnum en){opt = (int)en;}
        alertOption(){opt = -1;}

        operator int() const {return opt;}
        operator AlertOptions::alertOptionEnum() const {return (AlertOptions::alertOptionEnum)opt;}
        operator protocolOption() const {return protocolOption(opt);}

        alertOption & operator =(const AlertOptions::alertOptionEnum &en) {opt = (int)en;return *this;}
        bool operator == (const AlertOptions::alertOptionEnum &en) const {return ((int)en) == opt;}
        bool operator == (const alertOption &en) const {return en.opt == opt;}
};


uint qHash(const alertOption& opt);
uint qHash(const protocolOption& opt);

class alertManager;

class alertInfo
{
public:
        //Constructs clientInfo object from Kismet response,
        //fields (and their orders) specified by "options"
        alertInfo(kismetResponse &response, QSet<alertOption> &options);

        QString sec,usec,header,bssid,source,dest,other,channel,text,time;

        //Updates data in this object with data provided
        alertInfo& operator |= (const alertInfo &info);

protected:
        //Static map between client options and fields describing client preferences
        static QHash<alertOption,QString alertInfo::*> optionToMember;

private:
        //Method called in order to fill aforementioned map
        static void fillArray();
};

class alertManager : public dataManager
{
        Q_OBJECT
public:
        static alertManager* manager(QKismetSocket *socket);

        //Called when "response" received for "protocol"
        void parseData(const QString &protocol, kismetResponse &response);

        //Returns screen name for 'option'
        static QString optionName(alertOption option);
        QString optionName(protocolOption option);
        
        //Returns currectly enabled options
        const QSet<alertOption>& enabledOptions();

        //Retruns currently available options
        const QSet<protocolOption>& availableOptions();

        //Is specified option enabled ?
        bool isOptionEnabled(protocolOption opt);
signals:
        void data(alertInfo info);

public slots:
        void setOption(protocolOption option, bool enable = true);

private:
        alertManager(QKismetSocket *socket);

        //Called when user changed settings in order to send commands to kismet server
        void update();

        QKismetSocket *sock;

        //Lists with options.
        //options     - list managed by setOption = list for client protocol
        //avail       - same as available, bu with Time instead of Sec | Usec
        QSet<alertOption> options,optionsout;
        QSet<protocolOption> avail;

        //Associative arrays, needed for conversion between alerttOptions
        //and kismet protocol fields. Last one is used for conversion between alertOption
        //and screen name
        static QHash<alertOption,QString> optionToAlert,optionToName;

        void initArrays();

private slots:
        void socketReady();

        //Called when info about supported protocol fields received
        void capabilitiesChanged(const QString& protocol);

};

#endif // ALERTINFO_H
