/*
 * =====================================================================================
 *
 *	This file is part of qKismet, Qt Kismet GUI.
 *
 *	qKismet is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	qKismet is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * =====================================================================================
 *
 *       Filename:  clientinfo.cpp
 *
 *    Description:  Contains information about client from the server
 *
 *        Created:  2009-09-18 20:49:15 CEST
 *
 *	Copyright (C) 2009 Krzysztof Tyszecki <krzychoocpp@gmail.com>
 *
 * =====================================================================================
 */

#include "clientinfo.h"
#include "kisock.h"
#include "formattingutils.h"
#include <QDebug>

using namespace ClientOptions;

QHash<clientOption,QString clientInfo::*> clientInfo::optionToMember;


uint qHash(const clientOption& opt)
{
        return qHash((int)opt);
}


clientInfo::clientInfo(kismetResponse &response, QSet<clientOption> &options, clientManager* man)
{
        if(optionToMember.size() == 0)
                fillArray();

        foreach(clientOption opt, options)
                this->*optionToMember[opt]   = response.read().toString();

        //Saving this is not the best idea. But we need to know which options are enabled
        //in order to nicely format some data.
        source  = man;
}

void clientInfo::fillArray()
{
        optionToMember[Bssid]           = &clientInfo::bssid;
        optionToMember[Mac]             = &clientInfo::mac;
        optionToMember[Type]            = &clientInfo::type;
        optionToMember[Channel]         = &clientInfo::channel;
        optionToMember[Datapackets]     = &clientInfo::datapackets;
        optionToMember[Datasize]        = &clientInfo::datasize;
        optionToMember[Signaldbm]       = &clientInfo::signaldbm;
        optionToMember[Signalrssi]      = &clientInfo::signalrssi;
        optionToMember[Llcpackets]      = &clientInfo::llcpackets;
        optionToMember[Cryptpackets]    = &clientInfo::cryptpackets;
        optionToMember[Freq]            = &clientInfo::freq;
        optionToMember[Manuf]           = &clientInfo::manuf;
        optionToMember[GatewayIp]       = &clientInfo::gatewayip;
        optionToMember[Ip]              = &clientInfo::ip;

}

QString clientInfo::* clientInfo::optionMember(clientOption opt)
{
        return optionToMember[opt];
}

clientInfo& clientInfo::operator |=(const clientInfo &info)
{
        lastCh  = firstCh       = 0;
        int i = 0;
        foreach(clientOption opt,optionToMember.keys())
        {
                QString &oldv   = this->*optionToMember[opt];
                const QString &newv   = info.*optionToMember[opt];
                if(newv != "" && newv != oldv)
                {
                        if(firstCh == 0)
                                firstCh = i;
                        oldv       = newv;
                        lastCh  = i;
                }
                i++;
        }
        return *this;
}

QString clientInfo::prettyPrint(const clientOption &opt)
{
        QString value   = this->*optionToMember[opt];

        if(opt == Datasize)
                return formattingUtils::dataSize(value.toInt());
        if(opt == Type)
                return formattingUtils::networkType(value.toInt());
        if(opt == Freq)
        {
                int total       = -1;
                if(source->isOptionEnabled(Llcpackets) && source->isOptionEnabled(Datapackets))
                {
                        total   = (this->*optionToMember[Llcpackets]).toFloat();
                        total   += (this->*optionToMember[Datapackets]).toFloat();
                }
                formattingUtils::frequency(value,total);
        }
        return value;
}

int clientInfo::firstChanged()
{
        return firstCh;
}

int clientInfo::lastChanged()
{
        return lastCh;
}

QHash<clientOption,QString> clientManager::optionToClient;
QHash<clientOption,QString> clientManager::optionToName;

clientManager::clientManager(QKismetSocket *socket)
{
        sock    = socket;

        capabilitiesChanged("CLIENT");

        connect(sock->protoManager(),SIGNAL(capsChanged(QString)),this,SLOT(capabilitiesChanged(QString)));

        sock->registerProtocol(this,"CLIENT");

        initArrays();
}

clientManager* clientManager::manager(QKismetSocket *socket)
{
        clientManager *ret        = (clientManager*)socket->managerForProtocol("CLIENT");

        return (ret) ? ret : new clientManager(socket);
}

void clientManager::setOption(clientOption option, bool enable)
{
        if(option >= INVALID)
                return;
        if(enable && !options.contains(option))
        {
                options |= option;
                update();
        }
        else if(!enable && options.contains(option))
        {
                options.remove(option);
                update();
        }
}

void clientManager::setOption(protocolOption option, bool enable)
{
        setOption(clientOption(option),enable);
}

void clientManager::update()
{
        if(!sock->isWritable())
                return;

        sock->sendCommand(prepareCommand("CLIENT",options,clientManager::optionToClient));

        emit activeChanged();
}

QString clientManager::prepareCommand(QString protocol, QSet<clientOption> &options, QHash<clientOption,QString> &optionTo)
{
        QString cmd;

        //FIXME: it is possible that options.size() > 0 and available.size() == 0.


        enabled.clear();
        if(options.size() > 0)
        {
                cmd     += "ENABLE ";
                cmd     += protocol;
                cmd     += " ";

                QSet<clientOption>::const_iterator it = options.constBegin();
                while(it != options.constEnd())
                {
                        if(available.contains(*it))
                        {
                                cmd     += optionTo[*it];
                                cmd     += ',';
                                enabled |= *it;
                        }
                        it++;
                }
                cmd.chop(1);
        }
        else
        {
                cmd     += "REMOVE ";
                cmd     += protocol;
        }
        qDebug() << cmd;
        return cmd;
}

void clientManager::initArrays()
{
        if(optionToClient.size() == 0)
        {
                //BSSID protocol
                optionToClient[Bssid]            = "bssid";
                optionToClient[Mac]              = "mac";
                optionToClient[Type]             = "type";
                optionToClient[Channel]          = "channel";
                optionToClient[Datapackets]      = "datapackets";
                optionToClient[Datasize]         = "datasize";
                optionToClient[Llcpackets]       = "llcpackets";
                optionToClient[Freq]             = "freqmhz";
                optionToClient[GatewayIp]        = "gatewayip";
                optionToClient[Ip]               = "ip";

                //Screen names
                optionToName[Bssid]             = "BSSID";
                optionToName[Type]              = "client Type";
                optionToName[Mac]               = "MAC";
                optionToName[Channel]           = "Channel";
                optionToName[Datapackets]       = "Data packets";
                optionToName[Datasize]          = "Data size";
                optionToName[Llcpackets]        = "LLC packets";
                optionToName[Freq]              = "Frequency";
                optionToName[GatewayIp]         = "Gateway IP";
                optionToName[Ip]                = "IP";
        }
}

void clientManager::socketReady()
{
        update();
}

void clientManager::parseData(const QString &protocol, kismetResponse &response)
{
        if(protocol == "CLIENT")
                emit data(clientInfo(response,enabled,this));
}

QString clientManager::optionName(protocolOption option)
{
        return optionToName[option];
}

QString clientManager::optionName(clientOption option)
{
        return optionToName[option];
}

void clientManager::capabilitiesChanged(const QString& protocol)
{
        if(protocol == "CLIENT")
        {
                clientOption opt;
                clientFields   = sock->protoManager()->caps(protocol);

                available.clear();
                foreach(QString o,clientFields)
                if((opt = optionToClient.key(o,INVALID)) != INVALID)
                        available |= opt;

                update();
                emit dataManager::capabilitiesChanged();
        }
}

const QHash<clientOption,QString>& clientManager::optionNames()
{
        return optionToName;
}

const QSet<protocolOption>& clientManager::availableOptions()
{
        return available;
}

const QSet<clientOption>& clientManager::enabledOptions()
{
        return enabled;
}

bool clientManager::isOptionEnabled(protocolOption opt)
{
        return options.contains(opt);
}

clientOption clientManager::analogicalOption(networkOption opt)
{
        //TODO: hashmap
        switch((int)opt)
        {
                case NetworkOptions::Bssid:
                        return Mac;
                case NetworkOptions::Channel:
                        return Channel;
                case NetworkOptions::Cryptpackets:
                        return Cryptpackets;
                case NetworkOptions::Datapackets:
                        return Datapackets;
                case NetworkOptions::Datasize:
                        return Datasize;
                case NetworkOptions::Freq:
                        return Freq;
                case NetworkOptions::GatewayIp:
                        return GatewayIp;
                case NetworkOptions::Iprange:
                        return Ip;
                case NetworkOptions::Llcpackets:
                        return Llcpackets;
                case NetworkOptions::Manuf:
                        return Manuf;
                case NetworkOptions::Signaldbm:
                        return Signaldbm;
                case NetworkOptions::Type:
                        return Type;
        }
        return INVALID;
}
