/*
 * =====================================================================================
 *
 *	This file is part of qKismet, Qt Kismet GUI.
 *
 *	qKismet is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	qKismet is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * =====================================================================================
 *
 *       Filename:  clientinfo.h
 *
 *    Description:  Contains information about client from the server
 *
 *        Created:  2009-09-18 20:49:15 CEST
 *
 *	Copyright (C) 2009 Krzysztof Tyszecki <krzychoocpp@gmail.com>
 *
 * =====================================================================================
 */

#ifndef CLIENTINFO_H
#define CLIENTINFO_H

#include <QString>
#include <QHash>
#include "kismetresponse.h"
#include "protocolmanager.h"
#include "networkinfo.h"

class QKismetSocket;

namespace ClientOptions
{

enum clientOptionEnum{Bssid,Mac,Type,Channel,Datapackets,Datasize,Signaldbm,
            Signalrssi,Llcpackets,Cryptpackets,Freq,Manuf,GatewayIp,Ip,INVALID};
}



class clientOption : public protocolOption
{
        public:
                clientOption(const protocolOption &en){opt = (int)en;}
                clientOption(ClientOptions::clientOptionEnum en){opt = (int)en;}
                clientOption(){opt = -1;}

                operator int() const {return opt;}
                operator ClientOptions::clientOptionEnum() const {return (ClientOptions::clientOptionEnum)opt;}
                operator protocolOption() const {return protocolOption(opt);}

                clientOption & operator =(const ClientOptions::clientOptionEnum &en) {opt = (int)en;return *this;}
                bool operator == (const ClientOptions::clientOptionEnum &en) const {return ((int)en) == opt;}
                bool operator == (const clientOption &en) const {return en.opt == opt;}

};

uint qHash(const clientOption& opt);
uint qHash(const protocolOption& opt);

class clientManager;

//Class containing information about particular client
class clientInfo
{
public:
    //Constructs clientInfo object from Kismet response,
    //fields (and their orders) specified by "options"
    clientInfo(kismetResponse &response, QSet<clientOption> &options, clientManager* man);

    QString bssid,mac,type,channel,datasize,signaldbm,
            signalrssi,datapackets,llcpackets,cryptpackets,freq,manuf,gatewayip,ip;

    //Returns this client's opt property
    static QString clientInfo::* optionMember(clientOption opt);

    //Updates data in this object with data provided
    clientInfo& operator |= (const clientInfo &info);

    //Formats specified option for display
    QString prettyPrint(const clientOption &opt);

    int lastChanged();
    int firstChanged();
protected:

    //Static map between client options and fields describing client preferences
    static QHash<clientOption,QString clientInfo::*> optionToMember;
private:
    //Method called in order to fill aforementioned map
    static void fillArray();

    int lastCh,firstCh;
    //Source client manager
    clientManager *source;
};

//clientManager receives data from CLIENT protocol,
//and is used to set their options
class clientManager : public dataManager
{
        Q_OBJECT
public:
        static clientManager* manager(QKismetSocket *socket);

        //Called when "response" received for "protocol"
        void parseData(const QString &protocol, kismetResponse &response);

        //Returns screen name for 'option'
        static QString optionName(clientOption option);
        QString optionName(protocolOption option);


        //As above, but returns whole map
        const QHash<clientOption,QString>& optionNames();

        //Returns all available options
        const QSet<protocolOption>& availableOptions();

        //Returns currently enabled options
        const QSet<clientOption>& enabledOptions();

        //Is specified option enabled ?
        bool isOptionEnabled(protocolOption opt);

        //Returns client option analogical to opt
        //Eg. returns Mac if opt == Bssid
        static clientOption analogicalOption(networkOption opt);


signals:
        //Emitted when there's info about client
        void data(clientInfo info);

public slots:
        //Called in order to toggle 'option'
        void setOption(clientOption option, bool enable = true);

        //Provided for convenience
        void setOption(protocolOption option, bool enable = true);

private:
        clientManager(QKismetSocket *socket);
        //Called when user changed settings in order to send commands to kismet server
        void update();


        QKismetSocket *sock;

        //TODO: Maybe QFlags ?
        //Lists with options.
        //options     - list managed by setOption = list for client protocol
        //enabled     - list containing options that are set and available
        QSet<clientOption> options,enabled;


        //Associative arrays, needed for conversion between clientOptions
        //and Kismet protocol fields. Last one is used for conversion between clientOption
        //and screen name
        static QHash<clientOption,QString> optionToClient,optionToName;

        //List contatining fields supported by server
        QStringList clientFields;
        void initArrays();

        QString prepareCommand(QString protocol, QSet<clientOption> &options, QHash<clientOption,QString> &optionTo);

private slots:
        void socketReady();

        //Called when info about supported protocol fields received
        void capabilitiesChanged(const QString& protocol);

};

#endif // CLIENTINFO_H
