/*
 * =====================================================================================
 *
 *	This file is part of qKismet, Qt Kismet GUI.
 *
 *	qKismet is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	qKismet is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * =====================================================================================
 *
 *       Filename:  infoinfo.cpp
 *
 *    Description:  Contains information from INFO protcol
 *
 *        Created:  2009-09-20 12:30:13 CEST
 *
 *	Copyright (C) 2009 Krzysztof Tyszecki <krzychoocpp@gmail.com>
 *
 * =====================================================================================
 */

#include "infoinfo.h"
#include "kisock.h"

using namespace InfoOptions;

QHash<infoOption,QString infoInfo::*> infoInfo::optionToMember;
QHash<infoOption,QString> infoManager::optionToInfo;

uint qHash(const infoOption& opt)
{
        return qHash((int)opt);
}

infoInfo::infoInfo(kismetResponse &response, QSet<infoOption> &options)
{
        if(optionToMember.size() == 0)
                fillArray();

        foreach(infoOption opt, options)
                this->*optionToMember[opt]   = response.read().toString();
}

void infoInfo::fillArray()
{
        optionToMember[Networks]  = &infoInfo::networks;
        optionToMember[Packets]   = &infoInfo::packets;
        optionToMember[Crypt]     = &infoInfo::crypt;
        optionToMember[Noise]     = &infoInfo::noise;
        optionToMember[Dropped]   = &infoInfo::dropped;
        optionToMember[Rate]      = &infoInfo::rate;
        optionToMember[Filtered]  = &infoInfo::filtered;
        optionToMember[Clients]   = &infoInfo::clients;
        optionToMember[Llcpackets]        = &infoInfo::llcpackets;
        optionToMember[Datapackets]       = &infoInfo::datapackets;
        optionToMember[Numsources]        = &infoInfo::numsources;
        optionToMember[Numerrorsources]   = &infoInfo::numerrorsources;
        optionToMember[Weak]      = &infoInfo::weak;
        optionToMember[Signal]    = &infoInfo::signal;
}

infoManager::infoManager(QKismetSocket *socket)
{
        sock    = socket;
        capabilitiesChanged("INFO");

        connect(sock->protoManager(),SIGNAL(capsChanged(QString)),this,SLOT(capabilitiesChanged(QString)));

        sock->registerProtocol(this,"INFO");

        initArrays();
}

infoManager* infoManager::manager(QKismetSocket *socket)
{
        infoManager *ret        = (infoManager*)socket->managerForProtocol("INFO");

        return (ret) ? ret : new infoManager(socket);
}

void infoManager::setOption(protocolOption option, bool enable)
{
        if(option >= INVALID)
                return;
        if(enable && !options.contains(option))
        {
                options |= option;
                update();
        }
        else if(!enable && options.contains(option))
        {
                options.remove(option);
                update();
        }
}

void infoManager::parseData(const QString &protocol, kismetResponse &response)
{
        emit data(infoInfo(response,options));
}

void infoManager::update()
{
        if(!sock->isWritable())
                return;
        QString cmd;
        if(options.size() > 0)
        {
                cmd     = "ENABLE INFO ";

                QSet<infoOption>::const_iterator it = options.constBegin();
                while(it != options.constEnd())
                {
                        if(available.contains(*it))
                        {
                                cmd     += optionToInfo[*it];
                                cmd     += ',';
                        }
                        it++;
                }
                cmd.chop(1);
        }
        else
                cmd     = "REMOVE INFO";
        qDebug() << cmd;

        sock->sendCommand(cmd);
}

void infoManager::initArrays()
{
        optionToInfo[Networks]  = "networks";
        optionToInfo[Packets]   = "packets";
        optionToInfo[Crypt]     = "crypt";
        optionToInfo[Noise]     = "noise";
        optionToInfo[Dropped]   = "dropped";
        optionToInfo[Rate]      = "rate";
        optionToInfo[Filtered]  = "filtered";
        optionToInfo[Clients]   = "clients";
        optionToInfo[Llcpackets]        = "llcpackets";
        optionToInfo[Datapackets]       = "datapackets";
        optionToInfo[Numsources]        = "numsources";
        optionToInfo[Numerrorsources]   = "numerrorsources";
        optionToInfo[Weak]      = "weak";
        optionToInfo[Signal]    = "signal";
}

void infoManager::socketReady()
{
        update();
}

void infoManager::capabilitiesChanged(const QString& protocol)
{
        if(protocol == "INFO")
        {
                infoOption opt;
                QStringList fields   = sock->protoManager()->caps(protocol);

                available.clear();
                foreach(QString o,fields)
                if((opt = optionToInfo.key(o,INVALID)) != INVALID)
                        available |= opt;

                update();
                emit dataManager::capabilitiesChanged();
        }
}
