/*
 * =====================================================================================
 *
 *	This file is part of qKismet, Qt Kismet GUI.
 *
 *	qKismet is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	qKismet is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * =====================================================================================
 *
 *       Filename:  kisview.cpp
 *
 *    Description:  QKismet generic view 
 *
 *        Created:  2007-07-05 16:14:10 CEST
 *
 *	Copyright (C) 2007 Krzysztof Tyszecki <krzychoocpp@gmail.com>
 *
 * =====================================================================================
 */

#include "netview.h"
#include "settings.h"

#include <QHeaderView>
#include <QSettings>
#include <QFileDialog>
#include <QFile>
#include <QApplication>
#include <QClipboard>
#include <QSortFilterProxyModel>



QKismetTreeView::QKismetTreeView(QAbstractItemModel *mod)
{
	kmod	= (KismetItemModel*)mod;
	header()->setClickable(true);
	proxy.setSourceModel(mod);
#if QT_VERSION >= 0x040200
	setSortingEnabled(true);
	proxy.setSortRole(Qt::UserRole);
	proxy.setDynamicSortFilter(true);
#endif
//	proxy.setFilterKeyColumn(1);
	setModel(&proxy);
	
	
	popupMenu	= new QMenu();
	
        asave	= popupMenu->addAction(QIcon(":/img/document-save.png"),"Save...");
	connect(asave,SIGNAL(triggered()),this,SLOT(saveContents()));
	
        acopy	= popupMenu->addAction(QIcon(":/img/edit-copy.png"),"Copy");
	connect(acopy,SIGNAL(triggered()),this,SLOT(copyContents()));

        asel	= popupMenu->addAction(QIcon(":/img/edit-select-all.png"),"Select all");
	connect(asel,SIGNAL(triggered()),this,SLOT(selectAll()));

        aclear	= popupMenu->addAction(QIcon(":/img/edit-clear.png"),"Clear");
	connect(aclear,SIGNAL(triggered()),this,SLOT(cleared()));
	
	setContextMenuPolicy(Qt::CustomContextMenu);
	connect(this,SIGNAL(customContextMenuRequested(const QPoint&)),this,SLOT(showPopupMenu(const QPoint&)));
	asave->setDisabled(true);
	acopy->setDisabled(true);
	asel->setDisabled(true);
	aclear->setDisabled(true);


	setSelectionMode(QAbstractItemView::ExtendedSelection);
}

QKismetTreeView::~QKismetTreeView()
{
	delete popupMenu;	
	delete kmod;
}

void QKismetTreeView::rowsInserted(const QModelIndex& parent,int start, int end)
{
	QTreeView::rowsInserted(parent,start,end);
	asave->setDisabled(false);
	acopy->setDisabled(false);
	asel->setDisabled(false);
	aclear->setDisabled(false);
}

void QKismetTreeView::saveContents()
{
	QString fileName = QFileDialog::getSaveFileName(this,
			     tr("Save File"), "", "Text Files (*.log *.txt);;All Files (*.*)");
	if(fileName == "")
		return;
	QFile file(fileName);
	if(!file.open(QIODevice::WriteOnly | QIODevice::Truncate | QIODevice::Text))
		return;
	QTextStream out(&file);
	
	QList<int> cols;
	int j	= header()->count();
	for(int i = 0; i < j;i++)
		if(!header()->isSectionHidden(i))
			cols << i;
	
	getIndexData(QModelIndex(),false,&cols,out);
	file.close();
}

void QKismetTreeView::copyContents()
{
	QList<int> cols;
	int j 	= header()->count();
	for(int i = 0; i < j;i++)
		if(!header()->isSectionHidden(i))
			cols << i;
	QString out;
	getIndexData(QModelIndex(),true,&cols,out);
	QApplication::clipboard()->setText(out);
}

void QKismetTreeView::getIndexData(QModelIndex ix, bool selonly, QList<int> *cols, QTextStream& out)
{
	QAbstractItemModel *mod		= model();
	QItemSelectionModel *sel	= selectionModel();
	QModelIndex item;

	int k 	= cols->count();
	int j	= mod->rowCount(ix);
	for(int i = 0;i < j;i++)
	{
		item	= mod->index(i,0,ix);
		if(!selonly || sel->isSelected(item)) 
		{
			for(int j = 0; j < k;j++)
			{
				out	<< mod->data(mod->index(i,cols->at(j),ix)).toString();
				if(j != k-1)
					out	<< '\t';
			}
			out	<< '\n';
		}
		getIndexData(item,selonly,cols,out);
	}
}

void QKismetTreeView::getIndexData(QModelIndex ix, bool selonly, QList<int> *cols, QString& out)
{
	QAbstractItemModel *mod		= model();
	QItemSelectionModel *sel	= selectionModel();
	QModelIndex item;

	int k 	= cols->count();
	int j	= mod->rowCount(ix);
	for(int i = 0;i < j;i++)
	{
		item	= mod->index(i,0,ix);
		if(!selonly || sel->isSelected(item)) 
		{
			for(int j = 0; j < k;j++)
			{
				out	+= mod->data(mod->index(i,cols->at(j),ix)).toString();
				if(j != k-1)
					out	+= '\t';
			}
			out	+= '\n';
		}
		getIndexData(item,selonly,cols,out);
	}
}

QList<QAction*> QKismetTreeView::menuActions()
{
	return popupMenu->actions();
}

void QKismetTreeView::showPopupMenu(const QPoint& pos)
{
	popupMenu->exec(viewport()->mapToGlobal(pos));
}


void QKismetTreeView::selectionChanged(const QItemSelection& selected, const QItemSelection& deselected)
{
	QTreeView::selectionChanged(selected,deselected);
	acopy->setEnabled(selected.size() > 0);
}

void QKismetTreeView::cleared()
{
	kmod->clear();
	asave->setDisabled(true);
	acopy->setDisabled(true);
	asel->setDisabled(true);
	aclear->setDisabled(true);
}

KismetItemModel* QKismetTreeView::dataModel()
{
	return kmod;
}


