/*
 * =====================================================================================
 *
 *	This file is part of qKismet, Qt Kismet GUI.
 *
 *	qKismet is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	qKismet is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * =====================================================================================
 *
 *       Filename:  netmodel.cpp
 *
 *    Description:  Network view data model
 *
 *        Created:  2007-07-31 16:13:06 CEST
 *
 *	Copyright (C) 2007 Krzysztof Tyszecki <krzychoocpp@gmail.com>
 *
 * =====================================================================================
 */

#include "netmodel.h"
#include <QtAlgorithms>

NetworkModel::NetworkModel(networkManager *man, clientManager *cman) : KismetDataModel<NetworkRootItem,NetworkItem>()
{
        manager         = man;
        cmanager        = cman;
        colCount        = 0;

        connect(man,SIGNAL(data(networkInfo)),this,SLOT(netInfo(networkInfo)));
        connect(man,SIGNAL(capabilitiesChanged()),this,SLOT(updateColumns()));
        connect(man,SIGNAL(optionToggled(networkOption,bool)),this,SLOT(updateClientOptions(networkOption,bool)));
        connect(cman,SIGNAL(data(clientInfo)),this,SLOT(cliInfo(clientInfo)));

        updateColumns();
}

Qt::ItemFlags NetworkModel::flags( const QModelIndex & index ) const
{
	if(!index.isValid())
		 return 0;
	else return Qt::ItemIsDragEnabled | Qt::ItemIsEnabled | Qt::ItemIsSelectable;
}

QVariant NetworkModel::data(const QModelIndex &index, int role) const
{
        if(!index.isValid())
		return QVariant();
	if(role == Qt::TextColorRole)
	{
		NetmodelItem *item	= static_cast<NetmodelItem*>(index.internalPointer());
		if(item->hasColor)
			return item->color;
		else
			return QVariant();
	}
	else if(role == Qt::DisplayRole || role == Qt::UserRole)
	{
                NetmodelItem *item	= static_cast<NetmodelItem*>(index.internalPointer());
                QVariant data           = item->data(columnToOption[index.column()]);
                //if(index.column() == 0)
                //        data    = data.toString()    + "\ntest";
                //if(role == Qt::UserRole)
			return data;
                //else
                //	return NetworkOptions::print(&data,index.column());
	}
	return QVariant();
}

QVariant NetworkModel::headerData(int section, Qt::Orientation orientation,int role) const
{
        if(orientation == Qt::Horizontal && role == Qt::DisplayRole)
                return networkManager::optionName(columnToOption[section]);
        return QVariant();
}

QModelIndex NetworkModel::index(int row, int column, const QModelIndex &parent) const
{
        if(!hasIndex(row,column,parent))
		return QModelIndex();
	
	if(!parent.isValid())
	{
		NetworkItem *it	= rootItem->items[row];
		if(it)
			return createIndex(row,column,it);
	}
	else
	{
		NetworkItem *it = static_cast<NetworkItem*>(parent.internalPointer());
		if(it)
		{
			ClientItem *cit	= it->items[row];
			if(cit)
				return createIndex(row,column,cit);
		}
        }
	return QModelIndex();
}

QModelIndex NetworkModel::parent(const QModelIndex &index) const
{
        if(!index.isValid())
		return QModelIndex();

	NetmodelItem *childItem = static_cast<NetmodelItem*>(index.internalPointer());
	NetmodelItem *parentItem = (NetmodelItem*)childItem->parent();

	if (parentItem == (NetmodelItem*)rootItem)
		return QModelIndex();

	return createIndex(parentItem->row(), 0, parentItem);
}

int NetworkModel::columnCount(const QModelIndex&) const
{
        return colCount;
}

int NetworkModel::rowCount(const QModelIndex &parent) const
{

	if(!parent.isValid())
		return rootItem->items.size();
	NetmodelItem *item = static_cast<NetmodelItem*>(parent.internalPointer());
	return item->rowCount();
}

//Slot function called when information about network arrives
void NetworkModel::netInfo(networkInfo info)
{
	//Find network in the list
        NetworkItem* item	= findItem(info.bssid);

	//Network is already in the list
	if(item)
	{
                //NetworkItem is derived from networkInfo. networkInfo overloads |= opeator,
                //this line updates data in this object with data from 'info'.
                *item |= info;
                int r 	= item->row();
                emit dataChanged(createIndex(r,item->firstChanged(),item),createIndex(r,item->lastChanged(),item));
        }
	//Network isn't in the list, we must add new item
	else
	{
		int c	= rootItem->items.size();
		beginInsertRows(QModelIndex(),c,c);
		item	= new NetworkItem(&info,rootItem);

		rootItem->items.append(item);

		endInsertRows();

	}
}

void NetworkModel::cliInfo(clientInfo info)
{
        NetworkItem* item	= findItem(info.bssid);
	
	if(item)
	{
		//Found network...
		//Find this client
		ClientItem *cli	= 0;

		int j	= item->items.size();
		for(int i = 0;i < j;i++)
                        if(item->items[i]->mac == info.mac)
			{
				cli	=  item->items[i];
				break;
			}
		if(cli)
		{
                        *cli |= info;
                        int r 	= cli->row();
                        emit dataChanged(createIndex(r,cli->firstChanged(),cli),createIndex(r,cli->lastChanged(),cli));
		}
		else
		{
			int c	= item->items.size();
			beginInsertRows(createIndex(item->row(),0,item),c,c);
			item->addClient(&info);
			endInsertRows();
		}

        }
}
NetworkItem* NetworkModel::findItem(QString& bssid)
{
	int j	= rootItem->items.size();
	for(int i = 0;i < j;i++)
		if(rootItem->items[i]->bssid == bssid)
			return rootItem->items[i];
	return 0;
}

void NetworkModel::updateColumns()
{
        beginRemoveColumns(QModelIndex(),0,colCount-1);
        colCount        = 0;
        endRemoveColumns();

        columnToOption.clear();
        int i = 0;
        foreach(networkOption opt,manager->availableOptions())
                columnToOption[i++]     = opt;

        beginInsertColumns(QModelIndex(),0,i-1);
        colCount        = i;
        endInsertColumns();
}

void NetworkModel::updateClientOptions(networkOption option, bool enable)
{
        cmanager->setOption(clientManager::analogicalOption(option),enable);
}

const QHash<int,networkOption>& NetworkModel::columnOptions()
{
        return columnToOption;
}
