/*
 * =====================================================================================
 *
 *	This file is part of qKismet, Qt Kismet GUI.
 *
 *	qKismet is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	qKismet is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * =====================================================================================
 *
 *       Filename:  networkinfo.cpp
 *
 *    Description:  Contains information about network from the server
 *
 *        Created:  2009-07-06 14:36:10 CEST
 *
 *	Copyright (C) 2009 Krzysztof Tyszecki <krzychoocpp@gmail.com>
 *
 * =====================================================================================
 */

#include "networkinfo.h"
#include "formattingutils.h"
#include "kisock.h"
#include <QDebug>

using namespace NetworkOptions;

QHash<networkOption,QString networkInfo::*> networkInfo::optionToMember;


uint qHash(const networkOption& opt)
{
        return qHash((int)opt);
}

uint qHash(const protocolOption& opt)
{
        return qHash((int)opt);
}


networkInfo::networkInfo(kismetResponse &response, QSet<networkOption> &options, networkManager* man)
{
        if(optionToMember.size() == 0)
                fillArray();

        foreach(networkOption opt, options)
                this->*optionToMember[opt]   = response.read().toString();

        //Saving this is not the best idea. But we need to know which options are enabled
        //in order to nicely format some data.
        source  = man;
}

void networkInfo::fillArray()
{
        optionToMember[Bssid]           = &networkInfo::bssid;
        optionToMember[Ssid]            = &networkInfo::ssid;
        optionToMember[Type]            = &networkInfo::type;
        optionToMember[Crypted]         = &networkInfo::crypted;
        optionToMember[Channel]         = &networkInfo::channel;
        optionToMember[Datapackets]     = &networkInfo::datapackets;
        optionToMember[Datasize]        = &networkInfo::datasize;
        optionToMember[Beaconproc]      = &networkInfo::beaconproc;
        optionToMember[Signaldbm]       = &networkInfo::signaldbm;
        optionToMember[Signalrssi]      = &networkInfo::signalrssi;
        optionToMember[Llcpackets]      = &networkInfo::llcpackets;
        optionToMember[Cryptpackets]    = &networkInfo::cryptpackets;
        optionToMember[Freq]            = &networkInfo::freq;
        optionToMember[Manuf]           = &networkInfo::manuf;
        optionToMember[Country]         = &networkInfo::country;
        optionToMember[GatewayIp]       = &networkInfo::ip;
        optionToMember[Iprange]         = &networkInfo::iprange;


}

QString networkInfo::* networkInfo::optionMember(networkOption opt)
{
        return optionToMember[opt];
}

networkInfo& networkInfo::operator |=(const networkInfo &info)
{
        lastCh  = firstCh       = 0;
        int i = 0;
        foreach(networkOption opt,optionToMember.keys())
        {
                QString &oldv   = this->*optionToMember[opt];
                const QString &newv   = info.*optionToMember[opt];
                if(newv != "" && newv != oldv)
                {
                        if(firstCh == 0)
                                firstCh = i;
                        oldv       = newv;
                        lastCh  = i;
                }
                i++;
        }
        return *this;
}

int networkInfo::lastChanged()
{
        return lastCh;
}

int networkInfo::firstChanged()
{
        return firstCh;
}

QString networkInfo::prettyPrint(const networkOption &opt)
{
        QString value   = this->*optionToMember[opt];

        if(opt == Datasize)
                return formattingUtils::dataSize(value.toInt());
        if(opt == Crypted)
                return formattingUtils::networkEncryption(value.toInt());
        if(opt == Type)
                return formattingUtils::networkType(value.toInt());

        if(opt == Freq)
        {
                int total       = -1;
                if(source->isOptionEnabled(Llcpackets) && source->isOptionEnabled(Datapackets))
                {
                        total   = (this->*optionToMember[Llcpackets]).toFloat();
                        total   += (this->*optionToMember[Datapackets]).toFloat();
                }
                formattingUtils::frequency(value,total);
        }
        return value;
}

QHash<networkOption,QString> networkManager::optionToNetwork;
QHash<networkOption,QString> networkManager::optionToBssid;
QHash<networkOption,QString> networkManager::optionToSsid;
QHash<networkOption,QString> networkManager::optionToName;

networkManager::networkManager(QKismetSocket *socket)
{
        sock    = socket;
        ready   = 0;

        capabilitiesChanged("NETWORK");
        capabilitiesChanged("BSSID");
        capabilitiesChanged("SSID");

        connect(sock->protoManager(),SIGNAL(capsChanged(QString)),this,SLOT(capabilitiesChanged(QString)));

        sock->registerProtocol(this,"NETWORK");
        sock->registerProtocol(this,"BSSID");
        sock->registerProtocol(this,"SSID");

        initArrays();
}

networkManager* networkManager::manager(QKismetSocket *socket)
{
        networkManager *ret        = (networkManager*)socket->managerForProtocol("NETWORK");

        return (ret) ? ret : new networkManager(socket);
}

void networkManager::setOption(networkOption option, bool enable)
{
        if(option >= INVALID)
                return;
        if(enable && !network.contains(option))
        {
                network |= option;
                update();
                emit optionToggled(option,true);
        }
        else if(!enable && network.contains(option))
        {
                network.remove(option);
                update();
                emit optionToggled(option,false);
        }
}

void networkManager::setOption(protocolOption option, bool enable)
{
        setOption(networkOption(option),enable);
}

void networkManager::update()
{
        if(!sock->isWritable())
                return;
        if(sock->newcore())
        {
                //We are connected to newcore kismet server,
                //so we may need to send two commands
                //(and receive two replies)
                bssid.clear();
                ssid.clear();
                foreach(networkOption opt, network)
                {
                        switch((networkOptionEnum)opt)
                        {
                                case Bssid:
                                        bssid   << Bssid; break;
                                case Ssid:
                                        ssid    << Ssid; break;
                                case Type:
                                        bssid    << Type; break;
                                case Crypted:
                                        ssid    << Crypted; break;
                                case Channel:
                                        bssid   << Channel; break;
                                case Datapackets:
                                        bssid   << Datapackets; break;
                                case Datasize:
                                        bssid   << Datasize; break;
                                case Llcpackets:
                                        bssid   << Llcpackets; break;
                                case Freq:
                                        bssid   << Freq; break;
                                case GatewayIp:
                                        bssid   << GatewayIp; break;
                                case Iprange:
                                        bssid   << Iprange; break;
                        }
                }

                //We need bssid if we want something else
                if(bssid.size() > 0 && !bssid.contains(Bssid))
                        bssid |= Bssid;

                if(ssid.size() > 0)
                        ssid  |= Bssid;

                //Send commands to server
                sock->sendCommand(prepareCommand("BSSID", bssid,networkManager::optionToBssid));
                sock->sendCommand(prepareCommand("SSID", ssid,networkManager::optionToSsid));

        }
        else
        {
                sock->sendCommand(prepareCommand("NETWORK",network,networkManager::optionToNetwork));
        }
        emit activeChanged();
}

QString networkManager::prepareCommand(QString protocol, QSet<networkOption> &options, QHash<networkOption,QString> &optionTo)
{
        QString cmd;

        //FIXME: it is possible that options.size() > 0 and available.size() == 0.


        if(options.size() > 0)
        {
                cmd     += "ENABLE ";
                cmd     += protocol;
                cmd     += " ";

                QSet<networkOption>::const_iterator it = options.constBegin();
                while(it != options.constEnd())
                {
                        if(available.contains(*it))
                        {
                                cmd     += optionTo[*it];
                                cmd     += ',';
                        }
                        it++;
                }
                cmd.chop(1);
        }
        else
        {
                cmd     += "REMOVE ";
                cmd     += protocol;
        }
        qDebug() << cmd;
        return cmd;
}

void networkManager::initArrays()
{
        if(optionToBssid.size() == 0)
        {
                //BSSID protocol
                optionToBssid[Bssid]            = "bssid";
                optionToBssid[Type]             = "type";
                optionToBssid[Channel]          = "channel";
                optionToBssid[Datapackets]      = "datapackets";
                optionToBssid[Datasize]         = "datasize";
                optionToBssid[Llcpackets]       = "llcpackets";
                optionToBssid[Freq]             = "freqmhz";
                optionToBssid[GatewayIp]        = "gatewayip";
                optionToBssid[Iprange]          = "rangeip";

                //SSID
                optionToSsid[Bssid]             = "mac";
                optionToSsid[Ssid]              = "ssid";
                optionToSsid[Crypted]           = "cryptset";

                //NETWORK
                optionToNetwork = optionToBssid;
                optionToNetwork.remove(Freq);
                optionToNetwork.remove(GatewayIp);
                optionToNetwork[Crypted]        = "wep";
                optionToNetwork[Ssid]           = "ssid";

                //Screen names
                optionToName[Bssid]             = "BSSID";
                optionToName[Type]              = "Network Type";
                optionToName[Ssid]              = "SSID";
                optionToName[Crypted]           = "Encryption";
                optionToName[Channel]           = "Channel";
                optionToName[Datapackets]       = "Data packets";
                optionToName[Datasize]          = "Data size";
                optionToName[Llcpackets]        = "LLC packets";
                optionToName[Freq]              = "Frequency";
                optionToName[GatewayIp]         = "Gateway IP";
                optionToName[Iprange]           = "IP range";
        }
}

void networkManager::socketReady()
{
        update();
}

void networkManager::parseData(const QString &protocol, kismetResponse &response)
{
        if(protocol == "BSSID")
                emit data(networkInfo(response,bssid,this));
        else if(protocol == "SSID")
                emit data(networkInfo(response,ssid,this));
        else if(protocol == "NETWORK")
                emit data(networkInfo(response,network,this));
}

QString networkManager::optionName(protocolOption option)
{
        return optionToName[option];
}

QString networkManager::optionName(networkOption option)
{
        return optionToName[option];
}

void networkManager::capabilitiesChanged(const QString& protocol)
{
        networkOption opt;

        if(protocol == "NETWORK")
        {
                networkFields   = sock->protoManager()->caps(protocol);
                ready   = 2;

                available.clear();
                foreach(QString o,networkFields)
                if((opt = optionToNetwork.key(o,INVALID)) != INVALID)
                        available |= opt;
        }
        else if(protocol == "BSSID")
        {
                bssidFields     = sock->protoManager()->caps(protocol);
                ready   = (ready == 2 || ready == 0) ? 1 : 2;
        }
        else if(protocol == "SSID")
        {
                ssidFields      = sock->protoManager()->caps(protocol);
                ready   = (ready == 2 || ready == 0) ? 1 : 2;
        }
        else
                return;
        if(ready == 2)
        {
                if(sock->newcore())
                {
                        available.clear();

                        foreach(QString o,bssidFields)
                        if((opt = optionToBssid.key(o,INVALID)) != INVALID)
                                available |= opt;

                        foreach(QString o,ssidFields)
                        if((opt = optionToSsid.key(o,INVALID)) != INVALID)
                                available |= opt;

                        //qDebug() << available;
                }
                update();
                ready   = 0;

                emit dataManager::capabilitiesChanged();
        }
}

const QHash<networkOption,QString>& networkManager::optionNames()
{
        return optionToName;
}

const QSet<protocolOption>& networkManager::availableOptions()
{
        return available;
}

const QSet<networkOption>& networkManager::enabledOptions()
{
        return network;
}

bool networkManager::isOptionEnabled(protocolOption opt)
{
        return network.contains(opt);
}
