/*
 * =====================================================================================
 *
 *	This file is part of qKismet, Qt Kismet GUI.
 *
 *	qKismet is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	qKismet is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * =====================================================================================
 *
 *       Filename:  networkinfo.h
 *
 *    Description:  Contains information about network from the server
 *
 *        Created:  2009-07-06 14:36:10 CEST
 *
 *	Copyright (C) 2009 Krzysztof Tyszecki <krzychoocpp@gmail.com>
 *
 * =====================================================================================
 */

#ifndef NETWORKINFO_H
#define NETWORKINFO_H

#include <QString>
#include <QHash>
#include "kismetresponse.h"
#include "protocolmanager.h"

class QKismetSocket;

namespace NetworkOptions
{

enum networkOptionEnum{Bssid,Ssid,Type,Crypted,Channel,Datapackets,Datasize,Beaconproc,Signaldbm,
            Signalrssi,Llcpackets,Cryptpackets,Freq,Manuf,Country,GatewayIp,Iprange,INVALID};
}


class networkOption : public protocolOption
{
        public:
                networkOption(const protocolOption &en){opt = (int)en;}
                networkOption(NetworkOptions::networkOptionEnum en){opt = (int)en;}
                networkOption(){opt = -1;}

                operator int() const {return opt;}
                operator NetworkOptions::networkOptionEnum() const {return (NetworkOptions::networkOptionEnum)opt;}
                operator protocolOption() const {return protocolOption(opt);}

                networkOption & operator =(const NetworkOptions::networkOptionEnum &en) {opt = (int)en;return *this;}
                bool operator == (const NetworkOptions::networkOptionEnum &en) const {return ((int)en) == opt;}
                bool operator == (const networkOption &en) const {return en.opt == opt;}

};

uint qHash(const networkOption& opt);
uint qHash(const protocolOption& opt);

class networkManager;

//Class containing information about particular network
class networkInfo
{
public:
    //Constructs networkInfo object from Kismet response,
    //fields (and their orders) specified by "options"
    networkInfo(kismetResponse &response, QSet<networkOption> &options, networkManager* man);

    QString bssid,ssid,type,crypted,channel,packets,datasize,beaconproc,signaldbm,
            signalrssi,datapackets,llcpackets,cryptpackets,freq,manuf,country,ip,iprange;

    //Returns this network's opt property
    static QString networkInfo::* optionMember(networkOption opt);

    //Updates data in this object with data provided
    networkInfo& operator |= (const networkInfo &info);

    //Formats specified option for display
    QString prettyPrint(const networkOption &opt);

    int lastChanged();
    int firstChanged();
protected:

    //Static map between network options and fields describing network preferences
    static QHash<networkOption,QString networkInfo::*> optionToMember;
private:
    //Method called in order to fill aforementioned map
    static void fillArray();

    int lastCh,firstCh;

    //Source network manager
    networkManager *source;
};

//networkManager receives data from NETWORK or BSSID and SSID protocols,
//and is used to set their options
class networkManager : public dataManager
{
        Q_OBJECT
public:
        static networkManager* manager(QKismetSocket *socket);

        //Called when "response" received for "protocol"
        void parseData(const QString &protocol, kismetResponse &response);

        //Returns screen name for 'option'
        static QString optionName(networkOption option);
        QString optionName(protocolOption option);
        

        //As above, but returns whole map
        const QHash<networkOption,QString>& optionNames();

        //Returns all available options
        const QSet<protocolOption>& availableOptions();

        //Returns currently enabled options
        const QSet<networkOption>& enabledOptions();

        //Is specified option enabled ?
        bool isOptionEnabled(protocolOption opt);
signals:
        //Emitted when there's info about network
        void data(networkInfo info); 

        //Emitted when option has been toggled
        void optionToggled(networkOption opt, bool enabled);

public slots:
        //Called in order to toggle 'option'
        void setOption(networkOption option, bool enable = true);

        //Provided for convenience
        void setOption(protocolOption option, bool enable = true);

private:
        networkManager(QKismetSocket *socket);

        //Called when user changed settings in order to send commands to kismet server
        void update();

        QKismetSocket *sock;

        //TODO: Maybe QFlags ?
        //Lists with options.
        //network     - list managed by setOption = list for NETWORK protocol (old Kismet)
        //              and also list containing all enabled options
        //bssid, ssid - lists generated from network list, used for newcore protocols  
        QSet<networkOption> network,bssid,ssid;


        //Associative arrays, needed for conversion between networkOptions
        //and kismet protocol fields. Last one is used for conversion between networkOption
        //and screen name, used by e.g. NetworkModel for column names
        static QHash<networkOption,QString> optionToNetwork,optionToBssid,optionToSsid,optionToName;

        //Lists contatining fields supported by server
        QStringList networkFields,bssidFields,ssidFields;
        void initArrays();

        QString prepareCommand(QString protocol, QSet<networkOption> &options, QHash<networkOption,QString> &optionTo);

        char ready;
private slots:
        void socketReady();

        //Called when info about supported protocol fields received
        void capabilitiesChanged(const QString& protocol);

};
#endif // NETWORKINFO_H
